journal_mappings = {'A&A': 'Astron.Astrophys.',
                    'A&amp;A': 'Astron.Astrophys.',
                    'ApJ': 'Astrophys.J.',
                    'ARA&A': 'Ann.Rev.Astron.Astrophys.',
                    'ARA&amp;A': 'Ann.Rev.Astron.Astrophys.',
                    'Applied Optics': 'Appl.Opt.',
                    'ApJS': 'Astrophys.J.Suppl.',
                    'BAAS': 'Bull.Am.Astron.Soc.',
                    'Physics Letters': 'Phys.Lett.'}

edp_test_record = 'data/sample_edpsciences_record.xml'
aps_test_record = 'data/sample_aps_record.xml'
pos_test_record = 'data/sample_pos_record.xml'
consyn_test_record = 'data/sample_consyn_record.xml'
ws_test_record = 'data/sample_ws_record.xml'
ws_erratum_test_record = 'data/sample_ws_erratum_record.xml'
aps_output = 'data/sample_aps_output.xml'
edp_output = 'data/sample_edp_output.xml'
consyn_output = 'data/sample_consyn_output.xml'
ws_output = 'data/sample_ws_output.xml'
ws_erratum_output = 'data/sample_ws_erratum_output.xml'
xmllint = 'data/sample_elsevier_issue_input.xml'
xmllint_output = 'data/sample_elsevier_issue_output.xml'
xmllint_resources = 'data/si520'
